/* rvex_device_main.c - the main program for the tta devices executing ocl kernels

   Copyright (c) 2016 Michal Babej / Tampere University of Technology

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
*/

/*
#include <malloc.h>
#include <stdlib.h>
#include <lwpr.h>
*/

#include "pocl_device.h"

#define __local__ __attribute__((address_space(0)))
#define __global__ __attribute__((address_space(3)))
#define __constant__ __attribute__((address_space(3)))

typedef volatile __global__ __kernel_exec_cmd kernel_exec_cmd;
typedef __global__ __kernel_metadata kernel_metadata;

/**
 * Executes the work groups of the kernel command.
 */
static void rvex_opencl_wg_execute(
    kernel_exec_cmd* cmd,
    void** args,
    int first_gidx, int last_gidx) {

    kernel_metadata *kernel = (kernel_metadata*)cmd->kernel;

    const int num_groups_x = cmd->num_groups[0];
    const int num_groups_y = (cmd->work_dim >= 2) ? (cmd->num_groups[1]) : 1;
    const int num_groups_z = (cmd->work_dim == 3) ? (cmd->num_groups[2]) : 1;

    struct pocl_context context;
    context.work_dim = cmd->work_dim;
    context.num_groups[0] = cmd->num_groups[0];
    context.num_groups[1] = cmd->num_groups[1];
    context.num_groups[2] = cmd->num_groups[2];
    context.global_offset[0] = cmd->global_offset[0];
    context.global_offset[1] = cmd->global_offset[1];
    context.global_offset[2] = cmd->global_offset[2];
    context.local_size[0] = cmd->local_size[0];
    context.local_size[1] = cmd->local_size[1];
    context.local_size[2] = cmd->local_size[2];

    for (unsigned gid_x = first_gidx; gid_x <= last_gidx; gid_x++) {
        for (unsigned gid_y = 0; gid_y < num_groups_y; gid_y++) {
            for (unsigned gid_z = 0; gid_z < num_groups_z; gid_z++) {
                context.group_id[0] = gid_x;
                context.group_id[1] = gid_y;
                context.group_id[2] = gid_z;
                kernel->work_group_func (args, &context);
            }
        }
    }
}

/**
 * Prepares a work group for execution and launches it.
 */
static void rvex_opencl_wg_launch(kernel_exec_cmd* cmd) {

    void* args[MAX_KERNEL_ARGS];
    kernel_metadata *kernel = (kernel_metadata*)cmd->kernel;

    int num_groups_x = cmd->num_groups[0];
    int i, first_gid_x, last_gid_x;

    cmd->status = POCL_KST_RUNNING;

    /* single thread version: execute all work groups in
       a single trampoline call as fast as possible.
       Do not create any threads. */
    for (int i = 0; i < kernel->num_args + kernel->num_locals; ++i) {
        args[i] = (void*)cmd->args[i];
    }

    rvex_opencl_wg_execute(cmd, args, 0, num_groups_x - 1);

    cmd->status = POCL_KST_FINISHED;
}

/* The shared kernel_command object using which the device is controlled. */

int main() {
    int work_dim = 1;
    size_t local_work_sizes[3] = {1, 0, 0};
    size_t global_work_sizes[3] = {2, 0, 0};
    volatile kernel_exec_cmd *next_command = (kernel_exec_cmd*)KERNEL_EXE_CMD_OFFSET;

    do {

        while (next_command->status != POCL_KST_READY)
            ;

        rvex_opencl_wg_launch(next_command);

    } while (1);

    return 0;
}
